from typing import Any, Callable, Optional

from azure.ai.ml.dsl import pipeline as pipeline_dec
from azure.ai.ml.entities import Component, PipelineJob
from typing_extensions import override

from ez_azml.cloud_runs.cloud_run import CloudRun, RunOutput
from ez_azml.cloud_runs.commands import CommandRun


class PipelineRun(CloudRun):
    """Cloud run that uses mldesigner pipelines.

    Args:
        experiment_name: experiment this pipeline is associated with
        commands: commands to use within the pipeline
        pipeline: function representing the pipeline
        dec_kwargs: kwargs for azure.ai.ml.dsl.pipeline decorator
    """

    def __init__(
        self,
        experiment_name: str,
        commands: dict[str, CommandRun],
        pipeline: Callable,
        dec_kwargs: Optional[dict[str, Any]] = None,
        register_components: bool = False,
        **kwargs,
    ) -> None:
        super().__init__(**kwargs)
        self.experiment_name = experiment_name
        if isinstance(commands, CommandRun):
            commands = [commands]
        if isinstance(commands, list):
            commands = {command.name: command for command in commands}
        self.commands = commands
        self.dec_kwargs = dec_kwargs or {}
        self.pipeline = pipeline
        self.register_components = register_components

    def _get_command_components(
        self, commands: dict[str, CommandRun]
    ) -> list[Component]:
        return [
            command.register(self.ml_client)
            if self.register_components
            else command.get_component()
            for command in commands.values()
        ]

    def _build_pipeline(
        self,
        pipeline: Callable,
        dec_kwargs: Optional[dict[str, Any]],
        components: list[Component],
    ):
        # Inject the components
        for component in components:
            pipeline.__globals__[component.name] = component
        dec_pipeline = pipeline_dec(**dec_kwargs)(pipeline)
        return dec_pipeline

    def _setup_dec_kwargs(self, dec_kwargs: dict[str, Any]):
        if self.compute:
            dec_kwargs["default_compute"] = dec_kwargs.get(
                "default_compute", self.compute.name
            )
        if self.environment:
            # TODO: check correct key
            dec_kwargs["environment"] = dec_kwargs.get("environment", self.environment)
        return dec_kwargs

    @override
    def register(self) -> PipelineJob:
        for command in self.commands.values():
            command.ml_client = command.ml_client or self.ml_client
        self.components = self._get_command_components(self.commands)
        self.dec_kwargs = self._setup_dec_kwargs(self.dec_kwargs)
        self.pipeline = self._build_pipeline(
            self.pipeline, self.dec_kwargs, self.components
        )
        self.pipeline_component = self.pipeline(**self.inputs, **self.outputs)
        # TODO: add tenacity
        self.ml_client.jobs.create_or_update(
            self.pipeline_component, experiment_name=self.experiment_name
        )
        return self.pipeline_component

    @override
    def run(self) -> RunOutput:
        pipe = self.register()
        return RunOutput(url=pipe.studio_url)



from azure.ai.ml.entities import PipelineJob
from ez_azml.cloud_runs import PipelineRun


def test_e2e_register(pipeline: PipelineRun):
    """Tests e2e mock register."""
    component = pipeline.register()
    assert isinstance(component, PipelineJob)



from unittest import mock

import pytest
from ez_azml.params import DockerParams  # Replace with the actual import path

from tests.utils import mock_env_vars, mock_os_getenv


@pytest.mark.parametrize(
    "kwargs",
    [
        ({}),
        (
            {
                "image_name": "custom_image",
                "registry": "custom_registry",
                "tag": "custom_tag",
                "username": "custom_username",
                "password": "custom_password",
            }
        ),
        ({"image_name": "custom_image", "registry": "custom_registry"}),
        ({"image_name": "custom_image", "tag": "custom_tag"}),
        ({"image_name": "custom_image", "registry": "", "tag": "custom_tag"}),
    ],
)
@mock.patch("os.getenv", side_effect=mock_os_getenv)
def test_docker_params(mock_os_getenv, kwargs):
    """Tests docker params constructor."""
    # Initialize DockerParams with the given kwargs
    params = DockerParams(**kwargs)

    # Derive expected values based on kwargs or mock_env_vars
    expected_image_name = kwargs.get("image_name", mock_env_vars["DOCKER_IMAGE"])
    expected_registry = kwargs.get("registry", mock_env_vars["DOCKER_REGISTRY"])
    expected_tag = kwargs.get("tag", mock_env_vars["DOCKER_TAG"])
    expected_username = kwargs.get("username", mock_env_vars["DOCKER_USERNAME"])
    expected_password = kwargs.get("password", mock_env_vars["DOCKER_PASSWORD"])

    # Build the expected image string
    if expected_registry:
        expected_image = f"{expected_registry}/{expected_image_name}:{expected_tag}"
    else:
        expected_image = f"{expected_image_name}:{expected_tag}"

    # Assertions
    assert params.image_name == expected_image_name
    assert params.registry == expected_registry
    assert params.tag == expected_tag
    assert params.username == expected_username
    assert params.password == expected_password
    assert params.image == expected_image



# Mock environment variables
from ez_azml import env

MOCK_DOCKER_IMAGE = "MOCK_DOCKER_IMAGE"
MOCK_DOCKER_REGISTRY = "MOCK_DOCKER_REGISTRY"
MOCK_DOCKER_TAG = "MOCK_DOCKER_TAG"
MOCK_DOCKER_USERNAME = "MOCK_DOCKER_USERNAME"
MOCK_DOCKER_PASSWORD = "MOCK_DOCKER_PASSWORD"  # noqa: S105
MOCK_AZURE_SUBSCRIPTION_ID = "MOCK_AZURE_SUBSCRIPTION_ID"
MOCK_AZURE_RESOURCE_GROUP_NAME = "MOCK_AZURE_RESOURCE_GROUP_NAME"
MOCK_AZURE_WORKSPACE = "MOCK_AZURE_WORKSPACE"

# Rebuild the dictionary using the new variables
mock_env_vars = {
    env.DOCKER_IMAGE: MOCK_DOCKER_IMAGE,
    env.DOCKER_REGISTRY: MOCK_DOCKER_REGISTRY,
    env.DOCKER_TAG: MOCK_DOCKER_TAG,
    env.DOCKER_USERNAME: MOCK_DOCKER_USERNAME,
    env.DOCKER_PASSWORD: MOCK_DOCKER_PASSWORD,
    env.AZURE_SUBSCRIPTION_ID: MOCK_AZURE_SUBSCRIPTION_ID,
    env.AZURE_RESOURCE_GROUP_NAME: MOCK_AZURE_RESOURCE_GROUP_NAME,
    env.AZURE_WORKSPACE: MOCK_AZURE_RESOURCE_GROUP_NAME,
}


# Mock env.os_field function to return environment variables
def mock_os_getenv(env_var, default=None):
    """Mocks getenv."""
    return mock_env_vars.get(env_var, default)



from unittest import mock

from ez_azml import EzMLClient
from ez_azml.env import (
    AZURE_RESOURCE_GROUP_NAME,
    AZURE_SUBSCRIPTION_ID,
    AZURE_WORKSPACE,
)

from tests.utils import mock_os_getenv


@mock.patch("os.getenv", side_effect=mock_os_getenv)
def test_ml_client_envs(mock_os_getenv):
    """Tests that init properly retrieves env variables."""
    client = EzMLClient()

    assert client.subscription_id == mock_os_getenv(AZURE_SUBSCRIPTION_ID)
    assert client.resource_group_name == mock_os_getenv(AZURE_RESOURCE_GROUP_NAME)
    assert client.workspace_name == mock_os_getenv(AZURE_WORKSPACE)



from .compute import Cluster, SpotCluster
from .environments import DockerEnvironment
from .workspace import DockerWorkspaceConnection

__all__ = ["SpotCluster", "Cluster", "DockerEnvironment", "DockerWorkspaceConnection"]



from pathlib import Path
from typing import Callable
from unittest import mock

import pytest
from azure.ai.ml import Input, MLClient, Output
from azure.ai.ml.entities import Environment
from ez_azml.cloud_runs import CommandRun, PipelineRun


@pytest.fixture()
def ml_client() -> MLClient:  # type: ignore
    """MLClient fixture."""
    mock_credential = mock.MagicMock()
    ml_client = MLClient(credential=mock_credential)
    with (
        mock.patch.object(ml_client.components, "create_or_update", return_value=True),
        mock.patch.object(ml_client.jobs, "create_or_update", return_value=True),
        mock.patch.object(
            ml_client.environments, "create_or_update", return_value=True
        ),
    ):
        yield ml_client


@pytest.fixture()
def conda_env_file() -> Path:
    """Path to a conda env file."""
    return Path(__file__).parent.parent / "configs/environments/conda.yaml"


@pytest.fixture()
def environment(conda_env_file: Path) -> Environment:
    """Environment fixture."""
    return Environment(
        image="mcr.microsoft.com/azureml/openmpi4.1.0-ubuntu20.04",
        name="test_environment",
        conda_file=str(conda_env_file),
    )


@pytest.fixture()
def command_code_path(tmp_path: Path) -> Path:
    """Points to tmp file."""
    path = tmp_path / "test_command.py"
    path.write_text("""
    print('this is a test command')
    """)
    return path


@pytest.fixture()
def command(ml_client: MLClient, command_code_path: Path, environment: Environment):
    """CommandRun fixture."""
    inputs = {"command_input": Input()}
    return CommandRun(
        ml_client=ml_client,
        code=command_code_path,
        commands="python command.py",
        environment=environment,
        inputs=inputs,
    )


@pytest.fixture()
def pipeline_fn() -> Callable:
    """Function used in azure.ai.ml pipelines."""

    def pipeline__fn(
        pipeline_input: Input,
        pipeline_output: Output,
    ):
        test_command(command_input=pipeline_input)  # type: ignore # noqa F821

    return pipeline__fn


@pytest.fixture()
def pipeline(ml_client: MLClient, pipeline_fn: Callable, command: CommandRun):
    """PipelineRun cloud run fixture."""
    return PipelineRun(
        ml_client=ml_client,
        experiment_name="fixture",
        commands=[command],
        pipeline=pipeline_fn,
    )



from pathlib import Path

import yaml
from azure.ai.ml.entities import CommandComponent
from ez_azml.cloud_runs import CommandRun


def test_get_component_yaml(command: CommandRun, command_code_path: Path):
    """Tests that the yaml is properly generated."""
    yaml_io = command._get_component_yaml_stream(**command.register_kwargs)
    expected_yaml = {
        "code": str(command_code_path),
        "command": "python command.py",
        "environment": "azureml:test_environment:1",
        "inputs": {"command_input": {"type": "uri_folder"}},
        "name": "test_command",
        "outputs": {},
    }
    assert expected_yaml == yaml.safe_load(yaml_io)


def test_e2e_register(command: CommandRun):
    """Tests e2e mock register."""
    component = command.register()
    assert isinstance(component, CommandComponent)



# %%
from typing import ClassVar

from azure.ai.ml.entities import AmlCompute


class Cluster(AmlCompute):
    """AzureML Compute resource with some aliases for size.

    See Cluster.size_aliases
    """

    size_aliases: ClassVar[dict] = {
        "T4": "Standard_NC4as_T4_v3",
        "V100": "Standard_NC6s_v3",
        "CPU": "Standard_D1_v2",
    }

    def __init__(self, size: str, idle_time_before_scale_down: int = 5, **kwargs):
        size = self.size_aliases.get(size, size)
        super().__init__(
            size=size,
            idle_time_before_scale_down=idle_time_before_scale_down,
            **kwargs,
        )


class SpotCluster(Cluster):
    """Cluster that can be preempted."""

    def __init__(self, **kwargs):
        super().__init__(
            tier="low_priority",
            **kwargs,
        )



# %%
from azure.ai.ml.entities import UsernamePasswordConfiguration, WorkspaceConnection


class DockerWorkspaceConnection(WorkspaceConnection):
    """Azureml connection used for docker environments."""

    def __init__(
        self, username: str, password: str, name: str = "docker_registry", **kwargs
    ):
        credentials = UsernamePasswordConfiguration(
            username=username,
            password=password,
        )
        super().__init__(
            name=name, type="container_registry", credentials=credentials, **kwargs
        )



from ez_azml.entities.compute import Cluster, SpotCluster
from ez_azml.entities.workspace import DockerWorkspaceConnection
from ez_azml.ml_client import EzMLClient

__all__ = ["EzMLClient", "SpotCluster", "Cluster", "DockerWorkspaceConnection"]



from unittest import mock

import pytest
from ez_azml.cli import EzAzureMLCLI


@pytest.mark.parametrize("command", ["run", "register"])
def test_cloud_run_hooks_call(command: str):
    """Tests that cli is calling pre and post hooks."""
    mock_cloud_run = mock.MagicMock()
    cli = EzAzureMLCLI(cloud_run=mock_cloud_run)

    getattr(cli, command)()

    expected = [f"on_{command}_start", command, f"on_{command}_end"]
    for method in expected:
        getattr(mock_cloud_run, method).assert_called_once()


def test_ml_client_set():
    """Tests that ml_client is injected."""
    mock_cloud_run = mock.MagicMock()
    mock_cloud_run.ml_client = None
    cli = EzAzureMLCLI(cloud_run=mock_cloud_run)
    assert cli.ml_client == mock_cloud_run.ml_client



from typing import Optional

from azure.ai.ml.entities import Environment

from ez_azml.params import DockerParams


class DockerEnvironment(Environment):
    """Enviromment for training that uses a docker image.

    Args:
        docker: docker image's parameters.
        name: environment's name.
        description: environment's description.
    """

    def __init__(
        self,
        docker: DockerParams,
        name: Optional[str] = None,
        description: Optional[str] = None,
        **kwargs,
    ):
        name = name or f"{docker.image_name}-env-{docker.tag.replace('.','_')}".replace(
            "/", "-"
        )
        description = (
            description or f"{docker.image_name} Docker Command Job environment"
        )
        super().__init__(
            image=docker.image, name=name, description=description, **kwargs
        )



import io
from pathlib import Path
from typing import Any, Optional, Union

import yaml
from azure.ai.ml import Input, MLClient, Output, command, load_component
from azure.ai.ml.entities import Command, UserIdentityConfiguration, WorkspaceConnection
from typing_extensions import override

from ez_azml.cloud_runs.cloud_run import CloudRun, RunOutput


class CommandRun(CloudRun):
    """Cloud run that is based on AzureML Commands.

    Args:
        code: location of the python scripts to use
        commands: commands to run on the cloud (e.g. `python my_script.py`)
        flags: flags to use with the last command.
        identity: credentials to use.
        name: command's name.
        register_kwargs: kwargs to use when registering component.
    """

    def __init__(
        self,
        code: Union[str, Path],
        commands: Union[str, list[str]],
        ws_connection: Optional[WorkspaceConnection] = None,
        flags: Optional[list[str]] = None,
        identity: Optional[UserIdentityConfiguration] = None,
        name: Optional[str] = None,
        register_kwargs: Optional[dict[str, Any]] = None,
        **kwargs,
    ) -> None:
        super().__init__(**kwargs)
        self.ws_connection = ws_connection
        self.identity = identity or UserIdentityConfiguration()
        if isinstance(commands, str):
            commands = [commands]
        if flags:
            commands[-1] += " " + " ".join(flags)
        self.commands = commands
        self.code = Path(code)
        self.name = name or self.code.stem
        self.register_kwargs = register_kwargs or {}

    @property
    def cli_command(self) -> str:
        """Actual cli command run on AzureML."""
        return ";".join(self.commands)

    @property
    def command(self) -> Command:
        """Runnable command."""
        return command(
            command=self.cli_command,
            code=self.code,
            environment=self.environment,
            compute=self.compute.name,
            inputs=self.inputs,
            outputs=self.outputs,
            identity=self.identity,
        )

    def _get_io_dict(
        self, ios: dict[str, Union[Input, Output]], keys: Optional[list[str]] = None
    ):
        ios_as_dict = {}
        keys = keys or ["type"]
        for key, io_obj in ios.items():
            io_d = dict(io_obj)
            ios_as_dict[key] = {k: io_d[k] for k in keys if io_d[k] is not None}
        return ios_as_dict

    def _get_component_yaml_stream(
        self, name: Optional[str] = None, environment: Optional[str] = None, **kwargs
    ) -> io.StringIO:
        inputs_dict = self._get_io_dict(self.inputs)
        outputs_dict = self._get_io_dict(self.outputs)
        if not environment:
            self.ml_client.environments.create_or_update(self.environment)
            version = self.environment.version or 1
            environment = f"azureml:{self.environment.name}:{version}"
        yaml_dict = {
            "name": name or self.name,
            "inputs": inputs_dict,
            "outputs": outputs_dict,
            "code": str(self.code),
            "command": self.cli_command,
            "environment": environment,
            **kwargs,
        }
        yaml_stream = io.StringIO()
        yaml.dump(yaml_dict, yaml_stream)
        yaml_stream.seek(0)  # Move the file pointer to the beginning
        return yaml_stream

    def get_component(self, **kwargs):
        """Returns the mldesigner component."""
        yaml_file = self._get_component_yaml_stream(**self.register_kwargs, **kwargs)
        return load_component(yaml_file)

    @override
    def register(self, ml_client: Optional[MLClient] = None, **kwargs):
        ml_client = ml_client or self.ml_client
        component = self.get_component(**kwargs)
        ml_client.components.create_or_update(component)
        return component

    @override
    def run(self) -> str:
        if self.ws_connection:
            self.ml_client.connections.create_or_update(self.ws_connection)
        self.ml_client.begin_create_or_update(self.compute).result()
        cloud_job = self.ml_client.create_or_update(self.command)
        return RunOutput(url=cloud_job.studio_url)



import os
from dataclasses import field
from typing import Optional

DOCKER_REGISTRY = "DOCKER_REGISTRY"
DOCKER_USERNAME = "DOCKER_USERNAME"
DOCKER_PASSWORD = "DOCKER_PASSWORD"  # noqa: S105
DOCKER_REGISTRY = "DOCKER_REGISTRY"
DOCKER_TAG = "DOCKER_TAG"
DOCKER_IMAGE = "DOCKER_IMAGE"

AZURE_SUBSCRIPTION_ID = "AZURE_SUBSCRIPTION_ID"
AZURE_RESOURCE_GROUP_NAME = "AZURE_RESOURCE_GROUP_NAME"
AZURE_WORKSPACE = "AZURE_WORKSPACE"


def os_field(env_var: str, default: Optional[str] = None) -> Optional[str]:
    """Dataclass field that retrieves an environment variable.

    Args:
        env_var: variable to try to retrive.
        default: value to use if env variable was not set
    """
    return field(default_factory=lambda: os.getenv(env_var, default))



from typing import Optional

from azure.ai.ml import MLClient
from azure.ai.ml.entities import AmlCompute
from loguru import logger

from ez_azml.cloud_runs.cloud_run import CloudRun
from ez_azml.ml_client import EzMLClient


class EzAzureMLCLI:
    """CLI class.

    Contains commands and interactions

    Args:
        cloud_run: object representing a cloud run
        ml_client: client used for cloud interactions.
        Will be injected on to the cloud_run if it's lacking one
        compute: cluster used to run the `cloud_run`.
        Will be injected on to the cloud_run if it's lacking one
    """

    def __init__(
        self,
        cloud_run: Optional[CloudRun] = None,
        ml_client: Optional[MLClient] = None,
        compute: Optional[AmlCompute] = None,
    ):
        self.cloud_run = cloud_run
        self.ml_client = ml_client or EzMLClient()
        self.compute = compute

        self.cloud_run.ml_client = self.cloud_run.ml_client or self.ml_client
        self.cloud_run.compute = self.cloud_run.compute or self.compute

    def run(self):
        """Runs the cloud run.

        Calls the object's `on_run_start` and `on_run_end` hooks
        """
        self.cloud_run.on_run_start()
        output = self.cloud_run.run()
        logger.info(f"Run available at {output.url}")
        self.cloud_run.on_run_end(output)

    def register(self):
        """Registers the cloud run as a reusable component."""
        self.cloud_run.on_register_start()
        self.cloud_run.register()
        self.cloud_run.on_register_end()
        logger.info("Cloud run registered!")



# %%
import os
from typing import Any, Optional

from azure.ai.ml import MLClient
from azure.core.credentials import TokenCredential
from azure.identity import DefaultAzureCredential

from ez_azml.env import (
    AZURE_RESOURCE_GROUP_NAME,
    AZURE_SUBSCRIPTION_ID,
    AZURE_WORKSPACE,
)


class EzMLClient(MLClient):
    """A easy client class to interact with Azure ML services.

    Args:
        credential: The credential to use for authentication.
            Defaults to DefaultAzureCredential()
        subscription_id: The Azure subscription ID.
            Defaults to env AZURE_SUBSCRIPTION_ID.
        resource_group_name: The Azure resource group.
            Defaults to env AZURE_RESOURCE_GROUP_NAME.
        workspace_name: The workspace to use in the client.
            Defaults to env AZURE_WORKSPACE.
    """

    def __init__(
        self,
        credential: Optional[TokenCredential] = None,
        subscription_id: Optional[str] = None,
        resource_group_name: Optional[str] = None,
        workspace_name: Optional[str] = None,
        **kwargs: Any,
    ) -> None:
        credential = credential or DefaultAzureCredential()
        subscription_id = subscription_id or os.getenv(AZURE_SUBSCRIPTION_ID)
        resource_group_name = resource_group_name or os.getenv(
            AZURE_RESOURCE_GROUP_NAME
        )
        workspace_name = workspace_name or os.getenv(AZURE_WORKSPACE)
        super().__init__(
            credential, subscription_id, resource_group_name, workspace_name, **kwargs
        )



from abc import ABC, abstractmethod
from dataclasses import dataclass
from typing import Optional

from azure.ai.ml import Input, MLClient, Output
from azure.ai.ml.entities import AmlCompute, Environment


@dataclass
class RunOutput:
    """Output info for a CloudRun."""

    url: str


class CloudRun(ABC):
    """Abstract class representing any process that runs on the cloud.

    Args:
        compute: Compute used to execute the run
        environment: Environment used to host the run
        ml_client: Client in charge of interacting with the cloud provider
    """

    def __init__(
        self,
        compute: Optional[AmlCompute] = None,
        environment: Optional[Environment] = None,
        ml_client: Optional[MLClient] = None,
        inputs: Optional[dict[str, Input]] = None,
        outputs: Optional[dict[str, Output]] = None,
    ) -> None:
        self.compute = compute
        self.environment = environment
        self.ml_client = ml_client
        self.inputs = inputs or {}
        self.outputs = outputs or {}

    def on_run_start(self):
        """Hook called before run is submitted."""
        return

    @abstractmethod
    def run(self) -> RunOutput:
        """Submits and runs the job."""

    def on_run_end(self, output: RunOutput):
        """Hook called once run has been submitted."""
        return

    def on_register_start(self):
        """Hook called before a component has been registered."""
        return

    @abstractmethod
    def register(self):
        """Registers the run as a reusable component."""

    def on_register_end(self):
        """Hook called after a component has been registered."""
        return



from dataclasses import dataclass

from ez_azml import env


@dataclass
class DockerParams:
    """Dataclass representing a docker setup.

    Args:
        image_name: name of the image, without registry or tag. Defaults to env
            DOCKER_IMAGE.
        registry: registry hosting the image. Defaults to env DOCKER_REGISTRY.
        tag: image tag. Defaults to env DOCKER_TAG if set, "latest" otherwise.
        username: username to log in to the registry. Defaults to env DOCKER_USERNAME.
        password: password to log in to the registry. Defaults to env DOCKER_PASSWORD.
    """

    image_name: str = env.os_field(env.DOCKER_IMAGE)  # noqa: RUF009
    registry: str = env.os_field(env.DOCKER_REGISTRY)  # noqa: RUF009
    tag: str = env.os_field(env.DOCKER_TAG, "latest")  # noqa: RUF009
    username: str = env.os_field(env.DOCKER_USERNAME)  # noqa: RUF009
    password: str = env.os_field(env.DOCKER_PASSWORD)  # noqa: RUF009

    @property
    def image(self) -> str:
        """Concatenated registry + image name + tag."""
        image = f"{self.image_name}:{self.tag}"
        if self.registry:
            image = f"{self.registry}/{image}"
        return image



from ez_azml.entities import DockerEnvironment, DockerWorkspaceConnection
from ez_azml.params import DockerParams

from .command import CommandRun


class DockerCommandRun(CommandRun):
    """A Command that uses a docker image.

    Args:
        docker: docker params representing the image to use
    """

    def __init__(self, docker: DockerParams, **kwargs) -> None:
        registry = docker.registry
        ws_connection = None
        if registry and docker.username and docker.password:
            ws_connection = DockerWorkspaceConnection(
                username=docker.username,
                password=docker.password,
                target=registry,
            )
        environment = DockerEnvironment(docker=docker)
        super().__init__(environment=environment, ws_connection=ws_connection, **kwargs)



from jsonargparse import CLI, ArgumentParser
from loguru import logger

from ez_azml.cli.ez_azml import EzAzureMLCLI


def main():
    """Actual cli command for the package."""
    try:
        from dotenv import load_dotenv

        logger.debug("package dotenv available. Calling dotenv.load_dotenv()")
        load_dotenv()
    except ImportError:
        pass
    CLI(
        EzAzureMLCLI,
        parser_class=ArgumentParser,
        env_prefix="EZAZML",
        default_env=True,
    )


if __name__ == "__main__":
    main()
